//!
//!
use super::{models, schema::member_sites};
use beteran_common_rust as bcr;
use diesel::prelude::*;
use diesel::result::Error;

///
pub struct Repository {}

impl std::fmt::Debug for Repository {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Repository of member_sites").finish()
  }
}

impl Default for Repository {
  fn default() -> Self {
    Self::new()
  }
}

impl Repository {
  ///
  pub fn new() -> Repository {
    Repository {}
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
  ) -> Result<models::MemberSite, Error> {
    member_sites::table
      .find(id as uuid::Uuid)
      .first::<models::MemberSite>(conn)
  }

  ///
  pub fn select_by_url(
    &self,
    conn: &diesel::PgConnection,
    url: &str,
  ) -> Result<Option<models::MemberSite>, Error> {
    use member_sites::dsl;

    match member_sites::table
      .filter(dsl::url.eq(url))
      .first::<models::MemberSite>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: models::FindAll,
  ) -> Result<i64, Error> {
    let mut q = member_sites::table.into_boxed();

    if let Some(sp) = find_all.url_like {
      q = q.filter(member_sites::dsl::url.like(sp));
    }

    q.count().get_result(conn)
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: models::FindAll,
  ) -> Result<Vec<models::MemberSite>, Error> {
    let mut q = member_sites::table.into_boxed();

    if let Some(sp) = find_all.url_like {
      q = q.filter(member_sites::dsl::url.like(sp));
    }

    if let Some(p) = find_all.pagination {
      let page = p.page.unwrap_or(1);

      if let Some(page_size) = p.page_size {
        q = q.offset(((page - 1) * page_size) as i64);
        q = q.limit(page_size as i64);
      }
    }
    if let Some(orderbys) = find_all.sorts {
      for s in orderbys {
        match s {
          bcr::models::pagination::Sort::ASC(property) => match property.as_str() {
            "url" => {
              q = q.order_by(member_sites::url.asc());
            }
            "created_at" => {
              q = q.order_by(member_sites::created_at.asc());
            }
            "updated_at" => {
              q = q.order_by(member_sites::updated_at.asc());
            }
            "deleted_at" => {
              q = q.order_by(member_sites::deleted_at.asc());
            }
            _ => {}
          },
          bcr::models::pagination::Sort::DESC(property) => match property.as_str() {
            "url" => {
              q = q.order_by(member_sites::url.desc());
            }
            "created_at" => {
              q = q.order_by(member_sites::created_at.desc());
            }
            "updated_at" => {
              q = q.order_by(member_sites::updated_at.desc());
            }
            "deleted_at" => {
              q = q.order_by(member_sites::deleted_at.desc());
            }
            _ => {}
          },
        };
      }
    }

    q.load::<models::MemberSite>(conn)
  }
}
